import { ApolloError } from "apollo-server-micro";
import { uniq } from "lodash";
import { callMedicalDb, generalInformationBody } from "../queryMedicalDb";
import { prescriptionConditionsWhitelist } from "../toolbox/constants";
import { DTOGeneralInformationResponse, GeneralInformations } from "../types";

export const getGeneralInformations = async ({
  shortname,
}: {
  shortname: string;
}): Promise<GeneralInformations> => {
  const { drugs } = (await callMedicalDb(generalInformationBody, {
    shortname,
  })) as unknown as DTOGeneralInformationResponse;
  
  if (drugs.length === 0) {
    throw new ApolloError(
      `Error: There is no drug with this shortname.`
    );
  }

  const ingredients = uniq(drugs.flatMap((drug) => drug.ingredients));
  const atcParent = getFirstDrugWithAtc(drugs, "atc_parent").atc_parent;
  const atc = getFirstDrugWithAtc(drugs, "atc").atc;

  const formDoseAndPrescriptionConditions = drugs.map(({ form, dose, conditions }) => {
    return {
      form,
      dose,
      prescriptionConditions: conditions?.icons ? conditions.icons
        .filter(({ alt }) => prescriptionConditionsWhitelist.includes(alt))
        .map(({ title }) => title) : []
    };
  });

  return {
    ingredients,
    atcParent,
    atc,
    formDoseAndPrescriptionConditions
  }
};

const getFirstDrugWithAtc = (
  drugs: DTOGeneralInformationResponse["drugs"],
  atc: "atc" | "atc_parent"
) => {
  return (
    drugs.find((drug) => drug[atc]?.code && drug.atc?.label) ||
    drugs.find((drug) => drug[atc]?.code) ||
    drugs.find((drug) => drug[atc]?.label) ||
    drugs[0]
  );
};
