import { ApolloError } from "apollo-server-micro";
import { capitalize, sortBy } from "lodash";
import { callMedicalDb, interactionsBody } from "../queryMedicalDb";
import { getInteractionSeverityValue, possibleNatureInteractions } from "../toolbox/interactionsMap";
import { DTOInteractionsResponse, InteractionFromDrugGuide } from "../types";

const possibleSources: string[] = ["Thesaurus - ANSM"];

export const getInteractions = async ({
  dci,
}: {
  dci: string;
}): Promise<InteractionFromDrugGuide[]> => {
  const response = (await callMedicalDb(interactionsBody, {
    dci,
  })) as unknown as DTOInteractionsResponse;

  if (response.dci === null) {
    throw new ApolloError(`Error: There is no drug with this dci.`);
  }

  return sortBy(
    response.dci.interactions
      .filter(
        ({ nature, details: { source } }) =>
          (!source || possibleSources.includes(source)) &&
          nature.some((n) => possibleNatureInteractions.includes(n))
      )
      .map(({ nature, with: _with, details }) => {
        return {
          severity: nature.map(getInteractionSeverityValue),
          with: _with.map((w) => w.dci).sort(),
          title: formatTitle(details.title),
          nature: details.nature,
          instructions: details.instructions,
          informations: details.informations,
          history: details.history,
          bibliography: details.bibliography,
        };
      }),
    (i) => i.title
  );
};

const formatTitle = (title: string): string => {
  const t = title.replace(/^(\+|-|\s)+|(\+|-|\s)+$/g, "");
  if (t.includes(" + ")) {
    return t
      .split(" + ")
      .map((c) => capitalize(c))
      .join(" + ");
  }

  if (t.includes(" - ")) {
    return t
      .split(" - ")
      .map((c) => capitalize(c))
      .join(" - ");
  }

  return capitalize(t);
};
