import { ApolloError } from "apollo-server-micro";
import { callMedicalDb, protocolsBody } from "../queryMedicalDb";
import {
  DTOConditions,
  DTOFirstInsuance,
  DTOPrescription,
  DTOProtocolsResponse,
  DTORenewal,
  FirstIssuance,
  FormDoseAndProtocols,
  Prescription,
  Protocols,
  Renewal,
} from "../types";

export const getProtocols = async ({
  shortname,
}: {
  shortname: string;
}): Promise<FormDoseAndProtocols[]> => {
  const { drugs } = (await callMedicalDb(protocolsBody, {
    shortname,
  })) as unknown as DTOProtocolsResponse;

  if (drugs.length === 0) {
    throw new ApolloError(`Error: There is no drug with this shortname.`);
  }

  return drugs
    .map(({ form, dose, conditions }) => {
      return {
        form,
        dose,
        protocols: getProtocolsFromConditions(conditions),
      };
    })
    .filter(({ protocols }) => !!protocols);
};

const getProtocolsFromConditions = (conditions: DTOConditions): Protocols => {
  if (!conditions) {
    return null;
  }

  const { uri, summary, prescription, first_issuance, renewal } = conditions;

  return {
    uri,
    summary,
    prescription: getPrescription(prescription),
    firstIssuance: getFirstInsuance(first_issuance),
    renewal: getRenewal(renewal),
  };
};

const getPrescription = (presciption?: DTOPrescription): Prescription => {
  if (
    !presciption ||
    Object.values(presciption).every((value) => value == null)
  ) {
    return null;
  }
  const {
    base_content,
    validity_initial_hospital_prescription,
    maximum_prescription_period,
    prescription_reserved,
    monitoring_during_treatment,
    hospital_prescription,
    exception_drugs,
    archives,
  } = presciption;
  return {
    baseContent: base_content,
    validityInitialHospitalPrescription: validity_initial_hospital_prescription,
    maximumPrescriptionPeriod: maximum_prescription_period,
    prescriptionReserved: prescription_reserved,
    monitoringDuringTreatment: monitoring_during_treatment,
    hospitalPrescription: hospital_prescription,
    exceptionDrugs: exception_drugs,
    archives,
  };
};

const getFirstInsuance = (firstIssuance?: DTOFirstInsuance): FirstIssuance => {
  if (
    !firstIssuance ||
    Object.values(firstIssuance).every((value) => value == null)
  ) {
    return null;
  }
  const { base_content, prescription_support, time_limit, duration_treatment } =
    firstIssuance;
  return {
    baseContent: base_content,
    prescriptionSupport: prescription_support,
    timeLimit: time_limit,
    durationTreatment: duration_treatment,
  };
};

const getRenewal = (renewal?: DTORenewal): Renewal => {
  if (!renewal || Object.values(renewal).every((value) => value == null)) {
    return null;
  }
  const { base_content, prescription_support } = renewal;
  return {
    baseContent: base_content,
    prescriptionSupport: prescription_support,
  };
};
