import { ApolloError } from "apollo-server-micro";
import { uniq } from "lodash";
import { callMedicalDb, specialtiesBody } from "../queryMedicalDb";
import { getCrushableWithoutBackslashes } from "../toolbox/crushabilityTools";
import {
  GuideContext,
  DTOSpecialtiesResponse,
  FormDoseAndSpecialties,
  DTOSpecialty,
  SpecialtyFromDrugGuide,
} from "../types";

export const getSpecialties = async ({
  shortname,
  context,
}: {
  shortname: string;
  context?: GuideContext;
}): Promise<FormDoseAndSpecialties[]> => {
  const { drugs } = (await callMedicalDb(
    specialtiesBody,
    {
      shortname,
    },
    context?.mdbHeaders || null
  )) as unknown as DTOSpecialtiesResponse;

  if (drugs.length === 0) {
    throw new ApolloError(`Error: There is no drug with this shortname.`);
  }

  return drugs.map(({ form, dose, specialties, formulary_specialties }) => {
    const cisInFormulary = formulary_specialties?.map(({ cis }) => cis) || [];
    return {
      form,
      dose,
      specialties: formatSpecialties(specialties, cisInFormulary),
    };
  });
};

export const formatSpecialties = (
  specialties: DTOSpecialty[],
  cisInFormulary: string[]
): SpecialtyFromDrugGuide[] => {
  return specialties.map(
    ({ name, cis, crushable, excipients, presentations }) => {
      return {
        name,
        excipients: uniq(excipients.map(({ name }) => name).sort()),
        crushable: getCrushableWithoutBackslashes(crushable),
        availableInFormulary: cisInFormulary.includes(cis),
        presentations: presentations.map(({ presentation, ...p }) => {
          return {
            ...p,
            name: presentation,
          };
        }),
      };
    }
  );
};
